@@ -149,17 +149,37 @@ static void deselect_graph_keys(bAnimContext *ac, short test, short sel, short d
 static int graphkeys_deselectall_exec(bContext *C, wmOperator *op)
 {
 	bAnimContext ac;
+	bAnimListElem *ale_active = NULL;
 	
 	/* get editor data */
 	if (ANIM_animdata_get_context(C, &ac) == 0)
 		return OPERATOR_CANCELLED;
-		
+	
+	/* find active F-Curve, and preserve this for later 
+	 * or else it becomes annoying with the current active
+	 * curve keeps fading out even while you're editing it
+	 */
+	ale_active = get_active_fcurve_channel(&ac);
+	
 	/* 'standard' behavior - check if selected, then apply relevant selection */
 	if (RNA_boolean_get(op->ptr, "invert"))
 		deselect_graph_keys(&ac, 0, SELECT_INVERT, TRUE);
 	else
 		deselect_graph_keys(&ac, 1, SELECT_ADD, TRUE);
 	
+	/* restore active F-Curve... */
+	if (ale_active) {
+		FCurve *fcu = (FCurve *)ale_active->data;
+		
+		/* all others should not be disabled, so we should be able to just set this directly... 
+		 * - selection needs to be set too, or else this won't work...
+		 */
+		fcu->flag |= (FCURVE_SELECTED | FCURVE_ACTIVE);
+		
+		MEM_freeN(ale_active);
+		ale_active = NULL;
+	}
+	
 	/* set notifier that things have changed */
 	WM_event_add_notifier(C, NC_ANIMATION | ND_KEYFRAME | NA_SELECTED, NULL);
 	
@@ -757,7 +777,7 @@ static void graphkeys_select_leftright(bAnimContext *ac, short leftright, short
 	if (leftright == GRAPHKEYS_LRSEL_LEFT) {
 		ked.f1 = MINAFRAMEF;
 		ked.f2 = (float)(CFRA + 0.1f);
-	} 
+	}
 	else {
 		ked.f1 = (float)(CFRA - 0.1f);
 		ked.f2 = MAXFRAMEF;
@@ -815,7 +835,7 @@ static int graphkeys_select_leftright_exec(bContext *C, wmOperator *op)
 	return OPERATOR_FINISHED;
 }
 
-static int graphkeys_select_leftright_invoke(bContext *C, wmOperator *op, wmEvent *event)
+static int graphkeys_select_leftright_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
 	bAnimContext ac;
 	short leftright = RNA_enum_get(op->ptr, "mode");
@@ -835,7 +855,7 @@ static int graphkeys_select_leftright_invoke(bContext *C, wmOperator *op, wmEven
 		UI_view2d_region_to_view(v2d, event->mval[0], event->mval[1], &x, NULL);
 		if (x < CFRA)
 			RNA_enum_set(op->ptr, "mode", GRAPHKEYS_LRSEL_LEFT);
-		else 	
+		else
 			RNA_enum_set(op->ptr, "mode", GRAPHKEYS_LRSEL_RIGHT);
 	}
 	
@@ -1049,10 +1069,7 @@ static tNearestVertInfo *get_best_nearest_fcurve_vert(ListBase *matches)
 	/* if list only has 1 item, remove it from the list and return */
 	if (matches->first == matches->last) {
 		/* need to remove from the list, otherwise it gets freed and then we can't return it */
-		nvi = matches->first;
-		BLI_remlink(matches, nvi);
-		
-		return nvi;
+		return BLI_pophead(matches);
 	}
 	
 	/* try to find the first selected F-Curve vert, then take the one after it */
@@ -1075,9 +1092,7 @@ static tNearestVertInfo *get_best_nearest_fcurve_vert(ListBase *matches)
 	/* if we're still here, this means that we failed to find anything appropriate in the first pass,
 	 * so just take the first item now...
 	 */
-	nvi = matches->first;
-	BLI_remlink(matches, nvi);
-	return nvi;
+	return BLI_pophead(matches);
 }
 
 /* Find the nearest vertices (either a handle or the keyframe) that are nearest to the mouse cursor (in area coordinates) 
@@ -1113,7 +1128,7 @@ static void mouse_graph_keys(bAnimContext *ac, const int mval[2], short select_m
 	nvi = find_nearest_fcurve_vert(ac, mval);
 	
 	/* check if anything to select */
-	if (nvi == NULL)	
+	if (nvi == NULL)
 		return;
 	
 	/* deselect all other curves? */
@@ -1241,7 +1256,7 @@ static void graphkeys_mselect_column(bAnimContext *ac, const int mval[2], short
 	nvi = find_nearest_fcurve_vert(ac, mval);
 	
 	/* check if anything to select */
-	if (nvi == NULL)	
+	if (nvi == NULL)
 		return;
 	
 	/* get frame number on which elements should be selected */
@@ -1297,7 +1312,7 @@ static void graphkeys_mselect_column(bAnimContext *ac, const int mval[2], short
 /* ------------------- */
 
 /* handle clicking */
-static int graphkeys_clickselect_invoke(bContext *C, wmOperator *op, wmEvent *event)
+static int graphkeys_clickselect_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
 	bAnimContext ac;
 	short selectmode;
@@ -1342,10 +1357,13 @@ void GRAPH_OT_clickselect(wmOperatorType *ot)
 	ot->idname = "GRAPH_OT_clickselect";
 	ot->description = "Select keyframes by clicking on them";
 	
-	/* api callbacks */
+	/* callbacks */
 	ot->invoke = graphkeys_clickselect_invoke;
 	ot->poll = graphop_visible_keyframes_poll;
 	
+	/* flags */
+	ot->flag = OPTYPE_UNDO;
+	
 	/* properties */
 	prop = RNA_def_boolean(ot->srna, "extend", 0, "Extend Select", ""); // SHIFTKEY
 	RNA_def_property_flag(prop, PROP_SKIP_SAVE);
